import { subtle } from 'node:crypto';
import {PRIVATE_TELEGRAM_BOT_ID, PRIVATE_TELEGRAM_PUBLIC_KEY} from "$env/static/private";

export const actions = {
    register: async ({ request }) => {
        const formData = await request.formData();
        if (!formData) return { success: false, error: "No valid form data was provided" };

        const initDataUrlEncoded = formData.get('init-data');
        if (!initDataUrlEncoded) return { success: false, error: "No valid form data was provided" };

        const eventId = formData.get('event-id');
        if (!eventId) return { success: false, error: "Missing event-id" };

        const dataArrayParams = new URLSearchParams(initDataUrlEncoded)
        const initDataArray = [...dataArrayParams];
        const signature = dataArrayParams.get('signature');
        if (!signature) return { success: false, error: "No signature was provided" };
        initDataArray.sort((d1, d2) => d1[0].charCodeAt(0) - d2[0].charCodeAt(0));

        const checkString = initDataArray.reduce((acc, currentValue) => {
            if (['signature', 'hash'].includes(currentValue[0])) return acc;
            acc.push(currentValue.join("="))
            return acc;
        }, [`${PRIVATE_TELEGRAM_BOT_ID}:WebAppData`]).join("\n")

        const algo = 'Ed25519';
        const key = await subtle.importKey("raw", Buffer.from(PRIVATE_TELEGRAM_PUBLIC_KEY, 'hex'), algo, false, ['verify']);
        const validSignature = await subtle.verify(algo, key, Buffer.from(signature, 'base64url'), Buffer.from(checkString));
        if (!validSignature) return { success: false, error: "Invalid signature" }

        return { success: true, error: null }
    }
}